import {
  renderMessages
} from "./chunk-LZJPMHAM.js";
import {
  LogLevel,
  Logger,
  formatMilliseconds,
  getTimestamp,
  useNamespaceFilter
} from "./chunk-CYX5OLOK.js";

// src/node/env.ts
import fs from "fs";
import path from "path";
var log = Logger("zeed:env");
var NEWLINE = "\n";
var RE_INI_KEY_VAL = /^\s*([\w_.-]+)\s*=\s*(.*)?\s*$/;
var RE_NEWLINES = /\\n/g;
var NEWLINES_MATCH = /\n|\r|\r\n/;
function parse(src, options = {}) {
  let obj = {};
  src.toString().split(NEWLINES_MATCH).forEach(function(line, idx) {
    const keyValueArr = line.match(RE_INI_KEY_VAL);
    if (keyValueArr != null) {
      const key = keyValueArr[1];
      let val = keyValueArr[2] || "";
      const end = val.length - 1;
      const isDoubleQuoted = val[0] === '"' && val[end] === '"';
      const isSingleQuoted = val[0] === "'" && val[end] === "'";
      if (isSingleQuoted || isDoubleQuoted) {
        val = val.substring(1, end);
        if (isDoubleQuoted) {
          val = val.replace(RE_NEWLINES, NEWLINE);
        }
      } else {
        val = val.trim();
      }
      obj[key] = val;
    } else {
      log.debug(`did not match key and value when parsing line ${idx + 1}: ${line}`);
    }
  });
  return obj;
}
function stringToPath(value) {
  return path.resolve(process.cwd(), value || ".");
}
function setupEnv(options = {}) {
  var _a, _b, _c, _d;
  const dotenvPath = (_b = options == null ? void 0 : options.path) != null ? _b : path.resolve(process.cwd(), (_a = options == null ? void 0 : options.filename) != null ? _a : ".env");
  const encoding = (_c = options == null ? void 0 : options.encoding) != null ? _c : "utf8";
  const debug = (options == null ? void 0 : options.debug) || false;
  log.active = debug;
  try {
    const parsedEnv = fs.existsSync(dotenvPath) ? parse(fs.readFileSync(dotenvPath, { encoding }), { debug }) : {};
    const parsedEnvLocal = fs.existsSync(dotenvPath + ".local") ? parse(fs.readFileSync(dotenvPath + ".local", { encoding }), { debug }) : {};
    const parsed = Object.assign({}, parsedEnv, parsedEnvLocal);
    let env = (_d = options == null ? void 0 : options.env) != null ? _d : process.env;
    Object.entries(parsed).forEach(([key, value]) => {
      if (typeof (options == null ? void 0 : options.prefix) === "string") {
        key = (options == null ? void 0 : options.prefix) + key;
      }
      if (!Object.prototype.hasOwnProperty.call(env, key)) {
        if (value != null) {
          log.info(`set env.${key} = ${value}`);
          env[key] = value;
        }
      } else {
        log.debug(`"${key}" is already defined and will not be overwritten`);
      }
    });
    return { parsed };
  } catch (e) {
    log.error(e);
    return { error: e };
  }
}

// src/node/filestorage.ts
import {
  readFileSync,
  writeFileSync,
  mkdirSync,
  rmSync,
  unlinkSync,
  readdirSync
} from "fs";
import { resolve } from "path";
var log2 = Logger("zeed:filestorage");
var FileStorage = class {
  constructor(basePath) {
    this.store = {};
    this.fileKeys = void 0;
    this.dirname = resolve(process.cwd(), basePath || ".fileStorage");
  }
  setItem(key, value) {
    const data = JSON.stringify(value);
    this.store[key] = data;
    try {
      const path2 = resolve(this.dirname, key + ".json");
      mkdirSync(this.dirname, { recursive: true });
      writeFileSync(path2, data, "utf8");
    } catch (err) {
      log2.error("setItem error", err);
    }
  }
  getItem(key) {
    if (this.store.hasOwnProperty(key)) {
      let value = this.store[key];
      return JSON.parse(value) || null;
    } else {
      try {
        const path2 = resolve(this.dirname, key + ".json");
        const data = readFileSync(path2, "utf8");
        if (data) {
          return JSON.parse(data);
        }
      } catch (err) {
        log2.error("getItem error", err);
      }
    }
    return null;
  }
  removeItem(key) {
    delete this.store[key];
    if (this.fileKeys != null) {
      const index = this.fileKeys.indexOf(key);
      if (index !== -1) {
        this.fileKeys.splice(index, 1);
      }
    }
    try {
      const path2 = resolve(this.dirname, key + ".json");
      unlinkSync(path2);
    } catch (err) {
    }
  }
  clear() {
    this.fileKeys = [];
    this.store = {};
    rmSync(this.dirname, { recursive: true, force: true });
  }
  allKeys() {
    if (this.fileKeys == null) {
      try {
        this.fileKeys = readdirSync(this.dirname, { withFileTypes: true }).filter((item) => !item.isDirectory() && item.name.endsWith(".json")).map((item) => item.name.slice(0, -5)) || [];
      } catch (err) {
      }
    }
    let keys = [...this.fileKeys || []];
    for (let key of Object.keys(this.store)) {
      if (!keys.includes(key)) {
        keys.push(key);
      }
    }
    keys.sort();
    return keys;
  }
};

// src/node/log-file.ts
import { createWriteStream, mkdirSync as mkdirSync2 } from "fs";
import { resolve as resolve2, dirname } from "path";
var namespaces = {};
function LoggerFileHandler(path2, opt = {}) {
  const {
    level = LogLevel.debug,
    colors: colors2 = true,
    levelHelper = false,
    nameBrackets = true,
    padding = 16,
    filter = void 0
  } = opt;
  path2 = resolve2(process.cwd(), path2);
  mkdirSync2(dirname(path2), { recursive: true });
  var stream = createWriteStream(path2, { flags: "a" });
  return (msg) => {
    if (msg.level < level)
      return;
    const time2 = new Date().toISOString();
    let name = msg.name || "";
    let ninfo = namespaces[name || ""];
    if (ninfo == null) {
      namespaces[name] = ninfo;
    }
    let args = [
      `[${name || "*"}]`,
      renderMessages(msg.messages, { pretty: false })
    ];
    function write(...args2) {
      stream.write(args2.join("	") + "\n");
    }
    switch (msg.level) {
      case LogLevel.info:
        write(time2, `I|*  `, ...args);
        break;
      case LogLevel.warn:
        write(time2, `W|** `, ...args);
        break;
      case LogLevel.error:
        write(time2, `E|***`, ...args);
        break;
      default:
        write(time2, `D|   `, ...args);
        break;
    }
  };
}

// src/node/log-node.ts
import tty from "tty";
var colors = [6, 2, 3, 4, 5, 1];
function selectColor(namespace) {
  let hash = 0;
  for (let i = 0; i < namespace.length; i++) {
    hash = (hash << 5) - hash + namespace.charCodeAt(i);
    hash |= 0;
  }
  return colors[Math.abs(hash) % colors.length];
}
var namespaces2 = {};
var time = getTimestamp();
var useColors = tty.isatty(process.stderr.fd);
function log3(...args) {
  return process.stderr.write(args.join(" ") + "\n");
}
function LoggerNodeHandler(opt = {}) {
  var _a;
  const {
    level = LogLevel.debug,
    colors: colors2 = true,
    levelHelper = true,
    nameBrackets = true,
    padding = 0,
    filter = void 0
  } = opt;
  const matches = useNamespaceFilter((_a = filter != null ? filter : process.env.ZEED) != null ? _a : process.env.DEBUG);
  return (msg) => {
    if (msg.level < level)
      return;
    if (!matches(msg.name))
      return;
    const timeNow = getTimestamp();
    let name = msg.name || "";
    let ninfo = namespaces2[name || ""];
    if (ninfo == null) {
      ninfo = {
        color: selectColor(name)
      };
      namespaces2[name] = ninfo;
    }
    const diff = formatMilliseconds(timeNow - time);
    let args;
    let displayName = nameBrackets ? `[${name}]` : name;
    if (padding > 0) {
      displayName = displayName.padStart(padding, " ");
    }
    if (colors2 && useColors) {
      const c = ninfo.color;
      const colorCode = "[3" + (c < 8 ? c : "8;5;" + c) + "m";
      args = [`${colorCode}${displayName}[0m | `];
      args.push(...msg.messages);
      args.push(`${colorCode}+${diff}[0m`);
    } else {
      args = [displayName, ...msg.messages];
      args.push(`+${diff}`);
    }
    switch (msg.level) {
      case LogLevel.info:
        if (levelHelper)
          args[0] = `I|*   ` + args[0];
        log3(...args);
        break;
      case LogLevel.warn:
        if (levelHelper)
          args[0] = `W|**  ` + args[0];
        log3(...args);
        break;
      case LogLevel.error:
        if (levelHelper)
          args[0] = `E|*** ` + args[0];
        log3(...args);
        break;
      default:
        if (levelHelper)
          args[0] = `D|    ` + args[0];
        log3(...args);
        break;
    }
  };
}

export {
  stringToPath,
  setupEnv,
  FileStorage,
  LoggerFileHandler,
  LoggerNodeHandler
};
//# sourceMappingURL=chunk-ACKVG77H.js.map
